@@ -1707,7 +1707,7 @@ gog_axis_populate_editor (GogObject *gobj,
 			     glade_xml_get_widget (gui, "axis_pref_box"),
 			     _("Scale"));
 
-	if (gog_axis_get_atype (axis) < GOG_AXIS_VIRTUAL) {
+	if (gog_object_is_visible (axis) && gog_axis_get_atype (axis) < GOG_AXIS_VIRTUAL) {
 	    /* Style page */
 	    (GOG_OBJECT_CLASS(parent_klass)->populate_editor) (gobj, editor, dalloc, cc);
 
@@ -2245,7 +2245,7 @@ gog_axis_view_padding_request (GogView *view,
 
 	label_padding.wr = label_padding.wl = label_padding.ht = label_padding.hb = 0;
 
-	axis_pos = GOG_AXIS_BASE (axis)->position;
+	axis_pos = gog_axis_base_get_clamped_position (GOG_AXIS_BASE (axis));
 
 	for (ptr = view->children; ptr != NULL ; ptr = ptr->next) {
 		child = ptr->data;
@@ -2254,16 +2254,26 @@ gog_axis_view_padding_request (GogView *view,
 			available.w = bbox->w;
 			available.h = bbox->h;
 			gog_view_size_request (child, &available, &req);
-			if (type == GOG_AXIS_X) 
-				if (axis_pos != GOG_AXIS_AT_HIGH)
-					label_padding.hb += req.h + pad_h;
-				else 
-					label_padding.ht += req.h + pad_h;
-			else  
-				if (axis_pos != GOG_AXIS_AT_HIGH)
-					label_padding.wl += req.w + pad_w;
-				else
-					label_padding.wr += req.w + pad_w;
+			if (type == GOG_AXIS_X)
+				switch (axis_pos) {
+					case GOG_AXIS_AT_HIGH:
+						label_padding.ht += req.h + pad_h;
+						break;
+					case GOG_AXIS_AT_LOW:
+					default:
+						label_padding.hb += req.h + pad_h;
+						break;
+				}
+			else
+				switch (axis_pos) {
+					case GOG_AXIS_AT_HIGH:
+						label_padding.wr += req.w + pad_w;
+						break;
+					case GOG_AXIS_AT_LOW:
+					default:
+						label_padding.wl += req.w + pad_w;
+						break;
+				}
 		}
 	}
 
@@ -2310,14 +2320,14 @@ gog_axis_view_size_allocate (GogView *view, GogViewAllocation const *bbox)
 	available.w = bbox->w;
 	available.h = bbox->h;
 
-	axis_pos = GOG_AXIS_BASE (axis)->position;
+	axis_pos = gog_axis_base_get_clamped_position (GOG_AXIS_BASE (axis));
 
 	for (ptr = view->children; ptr != NULL ; ptr = ptr->next) {
 		child = ptr->data;
 		pos = child->model->position;
 		if (IS_GOG_LABEL (child->model) && (pos & GOG_POSITION_MANUAL)) {
 			gog_view_size_request (child, &available, &req);
-			child_bbox = gog_object_get_manual_allocation (gog_view_get_model (child), 
+			child_bbox = gog_object_get_manual_allocation (gog_view_get_model (child),
 								       bbox, &req);
 			gog_view_size_allocate (child, &child_bbox);
 		} else {
@@ -2325,28 +2335,35 @@ gog_axis_view_size_allocate (GogView *view, GogViewAllocation const *bbox)
 				if (IS_GOG_LABEL (child->model)) {
 					gog_view_size_request (child, &available, &req);
 					if (type == GOG_AXIS_X) {
-						child_bbox.x = plot_area->x + 
+						child_bbox.x = plot_area->x +
 							(plot_area->w - req.w) / 2.0;
 						child_bbox.w = req.w;
 						child_bbox.h = req.h;
-						if (axis_pos != GOG_AXIS_AT_HIGH) {
-							child_bbox.y = tmp.y + tmp.h - req.h;
-						} else {
-							child_bbox.y = tmp.y;
-							tmp.y += req.h + pad_h;
+						switch (axis_pos) {
+							case GOG_AXIS_AT_HIGH:
+								child_bbox.y = tmp.y;
+								tmp.y += req.h + pad_h;
+								break;
+							case GOG_AXIS_AT_LOW:
+							default:
+								child_bbox.y = tmp.y + tmp.h - req.h;
+								break;
 						}
 						tmp.h -= req.h + pad_h;
-					} else { 
-						child_bbox.y = plot_area->y + 
+					} else {
+						child_bbox.y = plot_area->y +
 							(plot_area->h - req.h) / 2.0;
 						child_bbox.h = req.h;
 						child_bbox.w = req.w;
-						if (axis_pos != GOG_AXIS_AT_HIGH) {
-							child_bbox.x = tmp.x;
-							tmp.x += req.w + pad_w;
-						}
-						else {
-							child_bbox.x = tmp.x + tmp.w - req.w;
+						switch (axis_pos) {
+							case GOG_AXIS_AT_HIGH:
+								child_bbox.x = tmp.x + tmp.w - req.w;
+								break;
+							case GOG_AXIS_AT_LOW:
+							default:
+								child_bbox.x = tmp.x;
+								tmp.x += req.w + pad_w;
+								break;
 						}
 						tmp.w -= req.w + pad_w;
 					}
@@ -2355,7 +2372,7 @@ gog_axis_view_size_allocate (GogView *view, GogViewAllocation const *bbox)
 					gog_view_size_allocate (child, plot_area);
 				}
 			}
-		}	
+		}
 	}
 }
 
