@@ -88,6 +88,8 @@ struct _GraphGuruState {
 	GtkWidget   *delete_button;
 
 	FooCanvasItem	  *sample_graph_item;
+	FooCanvasItem	  *sample_graph_frame;
+	FooCanvasItem	  *sample_graph_shadow;
 
 	GtkContainer  	  *prop_container;
 	GtkTreeSelection  *prop_selection;
@@ -854,11 +856,23 @@ cb_sample_plot_resize (FooCanvas *canvas,
 	}
 	
 	foo_canvas_item_set (state->sample_graph_item,
-		"w", width,
-		"h", height,
+		"w", width > 2 ? width - 2 : 0,
+		"h", height > 2 ? height - 2 : 0,
 		"x", x,
 		"y", y,
 		NULL);
+	foo_canvas_item_set (state->sample_graph_frame,
+		"x2", x + width - 3,
+		"y2", y + height - 3,
+		"x1", x,
+		"y1", y,
+		NULL);
+	foo_canvas_item_set (state->sample_graph_shadow,
+		"x2", x + width - 1,
+		"y2", y + height - 1,
+		"x1", x + 3,
+		"y1", y + 3,
+		NULL);
 }
 
 static gboolean
@@ -930,18 +944,29 @@ graph_guru_init_format_page (GraphGuruState *s)
 	w = glade_xml_get_widget (s->gui, "sample-alignment");
 	canvas = foo_canvas_new ();
 	gtk_container_add (GTK_CONTAINER (w), canvas);
-	s->sample_graph_item = foo_canvas_item_new (
-		foo_canvas_root (FOO_CANVAS (canvas)), GOG_CONTROL_FOOCANVAS_TYPE,
-		"model", s->graph,
-		NULL);
+	s->sample_graph_shadow = foo_canvas_item_new (foo_canvas_root (FOO_CANVAS (canvas)),
+						      FOO_TYPE_CANVAS_RECT,
+						      "width_pixels", 2,
+						      "outline_color_rgba", 0xa0a0a0ff,	/* grey */
+						      NULL);
+	s->sample_graph_frame = foo_canvas_item_new (foo_canvas_root (FOO_CANVAS (canvas)),
+						     FOO_TYPE_CANVAS_RECT,
+						     "width_pixels", 1,
+						     "fill_color_rgba", 0xffffffff,	/* white */
+						     "outline_color_rgba", 0x707070ff,	/* grey */
+						     NULL);
+	s->sample_graph_item = foo_canvas_item_new (foo_canvas_root (FOO_CANVAS (canvas)),
+						    GOG_CONTROL_FOOCANVAS_TYPE,
+						    "model", s->graph,
+						    NULL);
 	gtk_widget_add_events (canvas, GDK_POINTER_MOTION_HINT_MASK);
 	cb_sample_plot_resize (FOO_CANVAS (canvas), &canvas->allocation, s);
 	g_signal_connect (G_OBJECT (canvas),
-		"size_allocate",
-		G_CALLBACK (cb_sample_plot_resize), s);
+			  "size_allocate",
+			  G_CALLBACK (cb_sample_plot_resize), s);
 	g_signal_connect_after (G_OBJECT (canvas),
-		"event",
-		G_CALLBACK (cb_canvas_select_item), s);
+				"event",
+				G_CALLBACK (cb_canvas_select_item), s);
 	gtk_widget_show (canvas);
 
 	/* Connect to selection-changed signal of graph view */
@@ -1247,15 +1272,17 @@ gog_guru_get_help_button (GtkWidget *guru)
  * @graph	: the graph to edit
  * @dalloc	: The data allocator to use for editing
  * @cc		: Where to report errors
- * @toplevel	: as a parent window.
  * @closure	:
  *
+ * CHANGED 0.5.3
+ * 	: drop the @toplevel window argument and have the callers handle
+ * 	  widget_show and set_transient
+ *
  * Returns: and shows new graph guru.
  **/
 GtkWidget *
 gog_guru (GogGraph *graph, GogDataAllocator *dalloc,
-	  GOCmdContext *cc, GtkWindow *toplevel,
-	  GClosure *closure)
+	  GOCmdContext *cc, GClosure *closure)
 {
 	int page = (graph != NULL) ? 1 : 0;
 	GraphGuruState *state;
@@ -1307,9 +1334,5 @@ gog_guru (GogGraph *graph, GogDataAllocator *dalloc,
 	g_object_set_data_full (G_OBJECT (state->dialog),
 		"state", state, (GDestroyNotify) graph_guru_state_destroy);
 
-#warning for 0.5.0 remove this and have the callers set transient and handle show, then drop the toplevel arg
-	go_gtk_nonmodal_dialog (toplevel, GTK_WINDOW (state->dialog));
-	gtk_widget_show (GTK_WIDGET (state->dialog));
-
 	return state->dialog;
 }
