@@ -1,3 +1,4 @@
+/* vim: set sw=8: -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  * go-locale.c :
  *
@@ -173,6 +174,26 @@ go_locale_get_currency (gboolean *precedes, gboolean *space_sep)
 	return lc_currency;
 }
 
+#if defined(G_OS_WIN32)
+static void
+go_locale_win32_get_user_default (GString *res, unsigned int id)
+{
+	GError *error = NULL;
+	WCHAR fmt_utf16[64];
+	int utf16_len = GetLocaleInfoW (LOCALE_USER_DEFAULT,
+		id, fmt_utf16, sizeof (fmt_utf16));
+	gsize  utf8_len;
+	char *fmt_utf8 = g_convert ((gchar *)fmt_utf16, utf16_len*2,
+		 "UTF-8", "UTF-16LE", NULL, &utf8_len, &error);
+	if (NULL != fmt_utf8)
+		g_string_append_len (lc_date_format, fmt_utf8, utf8_len);
+	else if (NULL != error) {
+		g_warning ("error: %s", error->message);
+		g_error_free (error);
+	}
+}
+#endif
+
 GString const *
 go_locale_get_date_format (void)
 {
@@ -182,7 +203,9 @@ go_locale_get_date_format (void)
 		else
 			lc_date_format = g_string_new (NULL);
 
-#ifdef HAVE_LANGINFO_H
+#if defined(G_OS_WIN32)
+		go_locale_win32_get_user_default (lc_date_format, LOCALE_SLONGDATE);
+#elif defined(HAVE_LANGINFO_H)
 		{
 			char const *fmt = nl_langinfo (D_FMT);
 			while (*fmt) {
@@ -220,16 +243,16 @@ go_locale_get_date_format (void)
 
 		/* Sanity check */
 		if (!g_utf8_validate (lc_date_format->str, -1, NULL)) {
-			g_warning ("Produced non-UTF-8 time format.  Please report.");
+			g_warning ("Produced non-UTF-8 date format.  Please report.");
 			g_string_truncate (lc_date_format, 0);
 		}
 
 		/* Default */
 		if (lc_date_format->len == 0) {
 			static gboolean warning = TRUE;
-			g_string_append (lc_date_format, "hh:mm:ss");
+			g_string_append (lc_date_format, "yyyy/mm/dd");
 			if (warning) {
-				g_warning ("Using default system time format: %s",
+				g_warning ("Using default system date format: %s",
 					   lc_date_format->str);
 				warning = FALSE;
 			}
@@ -249,7 +272,9 @@ go_locale_get_time_format (void)
 		else
 			lc_time_format = g_string_new (NULL);
 
-#ifdef HAVE_LANGINFO_H
+#if defined(G_OS_WIN32)
+		go_locale_win32_get_user_default (lc_date_format, LOCALE_STIME);
+#elif defined(HAVE_LANGINFO_H)
 		{
 			char const *fmt = nl_langinfo (T_FMT);
 			while (*fmt) {
@@ -308,45 +333,63 @@ go_locale_get_time_format (void)
  * go_locale_month_before_day :
  *
  * A quick utility routine to guess whether the default date format
- * uses day/month or month/day
+ * uses day/month or month/day.  Returns a value of the same meaning
+ * as go_format_month_before_day, i.e.,
+ *
+ * 0, if locale uses day before month
+ * 1, if locale uses month before day, unless the following applies
+ * 2, if locale uses year before month (before day)
  */
-gboolean
+int
 go_locale_month_before_day (void)
 {
-#ifdef HAVE_LANGINFO_H
-	static gboolean month_first = TRUE;
-
+	static int month_first = 1;
 	if (!date_order_cached) {
-		char const *ptr = nl_langinfo (D_FMT);
-
 		date_order_cached = TRUE;
-		month_first = TRUE;
-		if (ptr)
-			while (*ptr) {
+
+#if defined(G_OS_WIN32)
+		{
+			TCHAR str[2];
+			GetLocaleInfo (LOCALE_USER_DEFAULT, LOCALE_IDATE, str, 2);
+			month_first = str[0] != L'1';
+			/* FIXME: Figure out if year comes first.  */
+		}
+
+#elif defined(HAVE_LANGINFO_H)
+		{
+			char const *ptr = nl_langinfo (D_FMT);
+			while (ptr && *ptr) {
 				char c = *ptr++;
-				if (c == 'd' || c == 'D') {
-					month_first = FALSE;
+				switch (c) {
+				case 'd': case 'D': case 'e':
+					month_first = 0;
+					ptr = NULL;
 					break;
-				} else if (c == 'm')
+				case 'm': case 'b': case 'B': case 'h':
+					month_first = 1;
+					ptr = NULL;
 					break;
+				case 'C': case 'G': case 'g':
+				case 'y': case 'Y':
+					month_first = 2;
+					ptr = NULL;
+					break;
+				default: ;
+				}
 			}
+		}
+#else
+		{
+			static gboolean warning = TRUE;
+			if (warning) {
+				g_warning ("Incomplete locale library, dates will be month day year");
+				warning = FALSE;
+			}
+		}
+#endif
 	}
 
 	return month_first;
-#elif defined(G_OS_WIN32)
-	TCHAR str[2];
-
-	GetLocaleInfo (LOCALE_USER_DEFAULT, LOCALE_IDATE, str, 2);
-
-	return str[0] != L'1';
-#else
-	static gboolean warning = TRUE;
-	if (warning) {
-		g_warning ("Incomplete locale library, dates will be month day year");
-		warning = FALSE;
-	}
-	return TRUE;
-#endif
 }
 
 /* Use comma as the arg separator unless the decimal point is a
